import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
// ignore: import_of_legacy_library_into_null_safe
import 'package:pull_to_refresh/pull_to_refresh.dart';

/// 包含滚动动画的 grid
class XGridViewBuilder extends StatefulWidget {
  final int count; // grid 列数
  final List<Widget> children;
  final double childAspectRatio;
  final double? mainAxisSpacing;
  final double? crossAxisSpacing;

  final EdgeInsetsGeometry? padding;

  final ScrollPhysics physics; //滚动动画方式
  final bool shrinkWrap;
  final Axis scrollDirection; //item滚动方向
  final int maxLength; //显示动画的最大item条数

  final ScrollController? controller;

  /// 刷新相关
  final XListModel? xmodel;
  final bool enablePullUp;
  final bool enablePullDown;

  const XGridViewBuilder({
    Key? key,
    required this.count,
    required this.children,
    this.childAspectRatio = 1,
    this.mainAxisSpacing,
    this.crossAxisSpacing,
    this.physics = const BouncingScrollPhysics(),
    this.shrinkWrap = true,
    this.scrollDirection = Axis.vertical,
    this.maxLength = 0,
    this.padding,
    this.controller,
    this.xmodel,
    this.enablePullUp = true,
    this.enablePullDown = true,
  }) : super(key: key);

  @override
  XGridViewBuilderState createState() => XGridViewBuilderState();
}

class XGridViewBuilderState extends State<XGridViewBuilder> with TickerProviderStateMixin {
  AnimationController? animationController;

  @override
  void initState() {
    if (widget.maxLength > 0) {
      animationController = AnimationController(
        duration: const Duration(milliseconds: 1000),
        vsync: this,
      );
    }
    super.initState();
  }

  @override
  void dispose() {
    animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.xmodel != null) {
      return SmartRefresher(
        controller: widget.xmodel!.refreshController,
        header: const XRefreshHeader(),
        footer: const XRefresherFooter(),
        onRefresh: widget.xmodel!.refreshData,
        onLoading: widget.xmodel!.loadMore,
        enablePullDown: widget.enablePullDown,
        enablePullUp: widget.enablePullUp,
        child: _buildGridView(),
      );
    } else {
      return _buildGridView();
    }
  }

  GridView _buildGridView() {
    return GridView.count(
      crossAxisCount: widget.count,
      childAspectRatio: widget.childAspectRatio,
      controller: widget.controller,
      padding: widget.padding,
      shrinkWrap: widget.shrinkWrap,
      physics: widget.physics,
      scrollDirection: widget.scrollDirection,
      mainAxisSpacing: D.h(widget.mainAxisSpacing) ?? 0,
      crossAxisSpacing: D.h(widget.crossAxisSpacing) ?? 0,
      children: (widget.maxLength == 0 || animationController == null)
          ? widget.children
          : widget.children.map((item) {
              final index = widget.children.indexOf(item);
              return _XAnimateItem(
                index: index,
                widget: widget,
                animationController: animationController!,
              );
            }).toList(),
    );
  }
}

class _XAnimateItem extends StatelessWidget {
  const _XAnimateItem({
    required this.index,
    required this.widget,
    required this.animationController,
  });

  final int index;
  final XGridViewBuilder widget;
  final AnimationController animationController;

  @override
  Widget build(BuildContext context) {
    final int count =
        widget.children.length > widget.maxLength ? widget.maxLength : widget.children.length;
    final int animationIndex = index > count ? count : index;
    final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(
      CurvedAnimation(
        parent: animationController,
        curve: Interval(
          (1 / count) * animationIndex,
          1.0,
          curve: Curves.fastOutSlowIn,
        ),
      ),
    );
    animationController.forward();

    return AnimatedBuilder(
      animation: animationController,
      builder: (BuildContext context, Widget? child) {
        return FadeTransition(
          opacity: animation,
          child: Transform(
            transform: Matrix4.translationValues(
              widget.scrollDirection == Axis.vertical ? 0 : 100 * (1.0 - animation.value),
              widget.scrollDirection == Axis.vertical ? 50 * (1.0 - animation.value) : 0,
              0,
            ),
            child: widget.children[index],
          ),
        );
      },
    );
  }
}
