import 'package:flutter/material.dart';
import 'package:hgg_view/hgg_view.dart';
// ignore: import_of_legacy_library_into_null_safe
import 'package:pull_to_refresh/pull_to_refresh.dart';

/// 包含滚动动画的list
class XListViewBuilder extends StatefulWidget {
  final int itemCount; // item数量
  final IndexedWidgetBuilder itemBuilder; //item 构建
  final EdgeInsetsGeometry? padding;
  final ScrollController? controller;
  final Axis scrollDirection; //item滚动方向,默认:纵向
  final ScrollPhysics physics; //滚动动画方式,默认:类ios
  final bool shrinkWrap;
  final int maxLength; //显示动画的最大item条数,默认:3
  final Widget? separatorLine; //分割线控件

  final XListModel? xmodel; //传入 listmodel,开启刷新控件
  final bool enablePullUp; //上拉加载更多
  final bool enablePullDown; //下拉刷新

  const XListViewBuilder({
    Key? key,
    required this.itemCount,
    required this.itemBuilder,
    this.padding,
    this.controller,
    this.scrollDirection = Axis.vertical,
    this.physics = const BouncingScrollPhysics(),
    this.shrinkWrap = true,
    this.maxLength = 0,
    this.separatorLine,
    this.xmodel,
    this.enablePullUp = true,
    this.enablePullDown = true,
  }) : super(key: key);

  @override
  XListViewBuilderState createState() => XListViewBuilderState();
}

class XListViewBuilderState extends State<XListViewBuilder> with TickerProviderStateMixin {
  AnimationController? animationController;

  @override
  void initState() {
    if (widget.maxLength > 0) {
      animationController = AnimationController(
        duration: const Duration(milliseconds: 1000),
        vsync: this,
      );
    }
    super.initState();
  }

  @override
  void dispose() {
    animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.xmodel != null) {
      return SmartRefresher(
        controller: widget.xmodel!.refreshController,
        header: const XRefreshHeader(),
        footer: const XRefresherFooter(),
        onRefresh: widget.xmodel!.refreshData,
        onLoading: widget.xmodel!.loadMore,
        enablePullDown: widget.enablePullDown,
        enablePullUp: widget.enablePullUp,
        child: _buildListView(),
      );
    } else {
      return _buildListView();
    }
  }

  ListView _buildListView() {
    return ListView.separated(
        controller: widget.controller,
        padding: widget.padding,
        shrinkWrap: widget.shrinkWrap,
        physics: widget.physics,
        scrollDirection: widget.scrollDirection,
        itemCount: widget.itemCount,
        separatorBuilder: (BuildContext context, int index) {
          if (widget.maxLength == 0 || animationController == null) {
            return widget.separatorLine ?? G.empty;
          } else {
            return _XAnimateItem(
              index: index,
              widget: widget,
              animationController: animationController!,
              isItem: false,
            );
          }
        },
        itemBuilder: (context, index) {
          if (widget.maxLength == 0 || animationController == null) {
            return widget.itemBuilder(context, index);
          } else {
            // 对前[maxLength]个item依次渐隐过度显示
            return _XAnimateItem(
              index: index,
              widget: widget,
              animationController: animationController!,
            );
          }
        });
  }
}

class _XAnimateItem extends StatelessWidget {
  const _XAnimateItem({
    required this.index,
    required this.widget,
    required this.animationController,
    this.isItem = true,
  });

  final int index;
  final XListViewBuilder widget;
  final AnimationController animationController;
  final bool isItem;

  @override
  Widget build(BuildContext context) {
    final int count = widget.itemCount > widget.maxLength ? widget.maxLength : widget.itemCount;
    final int animationIndex = index > count ? count : index;
    final Animation<double> animation = Tween<double>(begin: 0.0, end: 1.0).animate(
      CurvedAnimation(
        parent: animationController,
        curve: Interval(
          (1 / count) * animationIndex,
          1.0,
          curve: Curves.fastOutSlowIn,
        ),
      ),
    );
    animationController.forward();

    return AnimatedBuilder(
      animation: animationController,
      builder: (BuildContext context, Widget? child) {
        return FadeTransition(
          opacity: animation,
          child: Transform(
            transform: Matrix4.translationValues(
              widget.scrollDirection == Axis.vertical ? 0 : 100 * (1.0 - animation.value),
              widget.scrollDirection == Axis.vertical ? 50 * (1.0 - animation.value) : 0,
              0,
            ),
            child: isItem ? widget.itemBuilder(context, index) : widget.separatorLine ?? G.empty,
          ),
        );
      },
    );
  }
}
