import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:hgg_view/src/theme/h_colors.dart';

/// 一个有趣的、实用的开关组件。支持设置提示、滑块装饰、阴影以及良好的交互。
class XSwitch extends StatefulWidget {
  final bool open; // 是否处于打开状态。默认 false。
  final ValueChanged<bool>? onChanged; // 当开关状态发生改变时，会回调该函数。
  final double? width; // 宽度。默认 59.23，符合美学
  final double? height; // 高度。默认会根据 [width] 进行计算，符合美学
  final double? offset; // 滑块与边缘的间距
  final Widget? openChild; // 打开状态的提示样式
  final Widget? closeChild; // 关闭状态的提示样式
  final double? childOffset; // 提示与边缘的间距
  final Color? color; // 关闭状态下的背景色
  final Color? openColor; // 打开状态下的背景色
  final Color? sliderColor; // 滑块颜色
  final Widget? sliderChild; // 滑块中的组件。超过范围会被剪裁。
  final bool enable; // 是否可用
  final Color? shadowColor; // 设置组件阴影颜色
  final Offset? shadowOffset; // 设置组件阴影偏移
  final double shadowBlur; // 设置组件高斯与阴影形状卷积的标准偏差。

  const XSwitch({
    Key? key,
    required this.onChanged,
    this.open = false,
    this.width = 59.23,
    this.height,
    this.offset,
    this.childOffset,
    this.closeChild,
    this.openChild,
    this.color,
    this.openColor,
    this.sliderColor,
    this.sliderChild,
    this.enable = true,
    this.shadowColor,
    this.shadowOffset,
    this.shadowBlur = 0.0,
  })  : assert(onChanged != null, "open and onChanged can't be None!"),
        super(key: key);

  @override
  State<StatefulWidget> createState() => _XSwitch();
}

class _XSwitch extends State<XSwitch> {
  late double fixOffset;
  bool draging = false;
  double dragDxW = 10.0;

  late bool open;
  double? offset;
  Color? color;
  Color? openColor;

  @override
  void initState() {
    super.initState();
    open = widget.open;
    offset = widget.offset;
    color = widget.color;
    openColor = widget.openColor;
    fixOffset = open
        ? widget.width! -
            (offset ?? 2.0 / 36.0 * (widget.height ?? widget.width! * 0.608)) * 2.0 -
            (widget.height ?? widget.width! * 0.608) * (32.52 / 36.0)
        : 0;
  }

  @override
  void didUpdateWidget(XSwitch oldWidget) {
    super.didUpdateWidget(oldWidget);
    fixOffset = open
        ? widget.width! -
            (offset ?? 2.0 / 36.0 * (widget.height ?? widget.width! * 0.608)) * 2.0 -
            (widget.height ?? widget.width! * 0.608) * (32.52 / 36.0)
        : 0;
  }

  @override
  Widget build(BuildContext context) {
    double height = widget.height ?? widget.width! * 0.608;
    double circleSize = (height * (32.52 / 36.0));
    offset = offset ?? 2.0 / 36.0 * height;
    double childOffset = widget.childOffset ?? height / 5.0;
    color = color ?? HColors.FF_CCCCCC;
    openColor = openColor ?? HColors.FF_FFC900;

    List<Widget?> children = [];

    /// background
    var showShadow = widget.shadowColor != null && widget.shadowBlur != 0;
    var background = AnimatedContainer(
      duration: const Duration(milliseconds: 350),
      decoration: BoxDecoration(
        color: (open ? openColor : color) ?? color,
        borderRadius: BorderRadius.all(Radius.circular(height / 2.0)),
        boxShadow: showShadow
            ? [
                BoxShadow(
                  color: widget.shadowColor!,
                  offset: widget.shadowOffset ?? const Offset(0, 0),
                  blurRadius: widget.shadowBlur,
                )
              ]
            : null,
      ),
      child: SizedBox(
        width: widget.width,
        height: height,
      ),
    );
    children.add(background);

    /// Prompt
    var showChild = open ? widget.openChild : widget.closeChild;
    if (showChild != null) {
      showChild = Positioned(
        left: open ? childOffset : null,
        right: open ? null : childOffset,
        child: showChild,
      );
      children.add(showChild);
    }

    /// slider
    var slider = AnimatedContainer(
      margin: EdgeInsets.fromLTRB(offset! + fixOffset, 0, 0, 0),
      duration: const Duration(milliseconds: 200),
      width: circleSize + (draging ? dragDxW : 0.0),
      child: Container(
        height: circleSize,
        clipBehavior: Clip.antiAlias,
        alignment: Alignment.center,
        decoration: BoxDecoration(
            color: widget.sliderColor ?? Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(circleSize / 2.0))),
        child: widget.sliderChild,
      ),
    );
    children.add(slider);

    /// When in an unavailable state, add a mask
    if (!widget.enable) {
      var disableMask = Opacity(
        opacity: 0.6,
        child: Container(
          width: widget.width,
          height: height,
          decoration: BoxDecoration(
              color: HColors.FF_F1F1F1,
              borderRadius: BorderRadius.all(Radius.circular(height / 2.0))),
        ),
      );
      children.add(disableMask);
    }

    return GestureDetector(
      onTap: widget.enable ? _handleOnTap : null,
      onHorizontalDragEnd: widget.enable ? _handleOnHorizontalDragEnd : null,
      onHorizontalDragUpdate: widget.enable ? _handleOnHorizontalDragUpdate : null,
      onHorizontalDragCancel: widget.enable ? _handleDragCancel : null,
      onHorizontalDragStart: widget.enable ? _handleDragStart : null,
      child: Stack(
        alignment: Alignment.centerLeft,
        children: children as List<Widget>,
      ),
    );
  }

  void _handleOnTap() {
    setState(() {
      open = !open;
      double height = widget.height ?? widget.width! * 0.608;
      double circleSize = (height * (32.52 / 36.0));
      if (open) {
        fixOffset = widget.width! - offset! - circleSize - offset!;
      } else {
        fixOffset = 0;
      }
      widget.onChanged!(open);
    });
  }

  void _handleDragStart(DragStartDetails details) {
    setState(() {
      draging = true;
    });
  }

  void _handleOnHorizontalDragUpdate(DragUpdateDetails details) {
    setState(() {
      double height = widget.height ?? widget.width! * 0.608;
      double circleSize = (height * (32.52 / 36.0));
      fixOffset = fixOffset + details.delta.dx;
      if (fixOffset < 0) {
        fixOffset = 0;
      } else if (fixOffset >
          widget.width! - offset! - circleSize - (draging ? dragDxW : 0.0) - offset!) {
        fixOffset = widget.width! - offset! - circleSize - (draging ? dragDxW : 0.0) - offset!;
      }
    });
  }

  void _handleOnHorizontalDragEnd(DragEndDetails details) {
    setState(() {
      draging = false;
      double height = widget.height ?? widget.width! * 0.608;
      double circleSize = (height * (32.52 / 36.0));
      double center =
          (widget.width! - offset! - circleSize - (draging ? dragDxW : 0.0) - offset!) / 2;
      bool cacheValue = open;
      if (fixOffset < center) {
        fixOffset = 0;
        open = false;
      } else {
        fixOffset = center * 2;
        open = true;
      }
      if (cacheValue != open) {
        widget.onChanged!(open);
      }
    });
  }

  void _handleDragCancel() {
    setState(() {
      draging = false;
    });
  }
}
